<?php

namespace Drupal\markdownify\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\markdownify\MarkdownifyEntityRendererInterface;
use Psr\Log\LoggerInterface;

/**
 * Service for rendering Drupal entities as HTML.
 *
 * This service provides functionality to render entities as HTML strings
 * using a specified view mode and language. The rendered HTML can also
 * be altered by other modules via the 'markdownify_entity_render' alter hook.
 */
class MarkdownifyEntityRenderer implements MarkdownifyEntityRendererInterface {

  /**
   * The entity type manager service.
   *
   * Used to load the appropriate view builder for rendering entities.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * The module handler service.
   *
   * Provides hooks for other modules to alter the rendered output.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Constructs a new MarkdownifyEntityRenderer object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, ModuleHandlerInterface $module_handler, LoggerInterface $logger) {
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
    $this->moduleHandler = $module_handler;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function toHtml(EntityInterface $entity, string $view_mode = 'full', ?string $langcode = NULL): string {
    try {
      // Obtain the view builder for the entity type.
      $view_builder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
      // Build the render array for the provided entity.
      $build = $view_builder->view($entity, $view_mode, $langcode);
      // Render the array into an isolated HTML string.
      $html = (string) $this->renderer->renderInIsolation($build);
      // Allow other modules to alter the rendered HTML.
      $context = [
        'entity' => $entity,
        'view_mode' => $view_mode,
        'langcode' => $langcode,
      ];
      $this->moduleHandler->alter('markdownify_entity_html', $html, $context);
      // Return the rendered output.
      return $html;
    }
    catch (\Exception $e) {
      // Log any exceptions that occur during rendering.
      $this->logger->error('Failed to render entity: @message', ['@message' => $e->getMessage()]);
      return '';
    }
  }

}
