<?php

namespace Drupal\markdownify\Service;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\markdownify\MarkdownifyHtmlConverterInterface;
use League\HTMLToMarkdown\HtmlConverter;
use Psr\Log\LoggerInterface;

/**
 * Service for converting HTML to Markdown.
 *
 * This service provides a mechanism to convert HTML content into Markdown
 * using the League HTML-to-Markdown library. The conversion process includes
 * options for stripping unsupported tags and handling line breaks.
 */
class MarkdownifyHtmlConverter implements MarkdownifyHtmlConverterInterface {

  /**
   * The module handler service.
   *
   * Provides hooks for other modules to alter the rendered output.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The logger service.
   *
   * Used to log any errors or warnings during the conversion process.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Constructs a new MarkdownifyHtmlConverter object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(ModuleHandlerInterface $module_handler, LoggerInterface $logger) {
    $this->moduleHandler = $module_handler;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function convert(string $html): string {
    if (empty($html)) {
      // Return an empty string if the input HTML is empty.
      return '';
    }
    try {
      // Initialize the HTML-to-Markdown converter with specific options.
      $converter = new HtmlConverter([
        'strip_tags' => TRUE,
        'hard_break' => FALSE,
        'header_style' => 'atx',
      ]);
      // Convert the HTML to Markdown.
      $markdown = $converter->convert($html);
      // Allow other modules to alter the generated Markdown.
      $context = ['html' => $html];
      $this->moduleHandler->alter('markdownify_entity_markdown', $markdown, $context);
      // Return the rendered output.
      return $markdown;
    }
    catch (\Exception $e) {
      // Log any exceptions that occur during the conversion process.
      $this->logger->error('Error converting HTML to Markdown: @message', ['@message' => $e->getMessage()]);
      return '';
    }
  }

}
